@@ -4,7 +4,7 @@ use tokio::{net::UdpSocket, sync::{Mutex, mpsc::channel, Notify}, time::{sleep,
 
 use tokio::sync::mpsc::{Sender, Receiver};
 use std::sync::atomic::{AtomicBool, Ordering};
-use crate::error::{Result, RaknetError};
+use crate::{error::{Result, RaknetError}};
 
 use crate::{packet::*, utils::*, arq::*, raknet_log};
 
@@ -121,7 +121,7 @@ impl RaknetSocket {
     async fn sendto(s : &UdpSocket , buf : &[u8] , target : &SocketAddr , enable_loss : &AtomicBool , loss_rate : &AtomicU8) -> tokio::io::Result<usize>{
         if enable_loss.load(Ordering::Relaxed){
             let mut rng = rand::thread_rng();
-            let i: u8 = rng.gen_range(1..11);
+            let i: u8 = rng.gen_range(0..11);
             if i > loss_rate.load(Ordering::Relaxed) {
                 raknet_log!("loss packet");
                 return Ok(0);
@@ -240,7 +240,14 @@ impl RaknetSocket {
                     Err(_) => continue
                 }{
                     Ok(p) => p,
-                    Err(_) => {
+                    Err(e) => {
+                        #[cfg(target_family = "windows")]
+                        if e.raw_os_error().unwrap() == 10040{
+                            // https://docs.microsoft.com/zh-CN/troubleshoot/windows-server/networking/wsaemsgsize-error-10040-in-winsock-2
+                            raknet_log!("recv_from error : {}" , e.raw_os_error().unwrap());
+                            continue;
+                        }
+                        raknet_log!("recv_from error : {}" , e);
                         connected_s.store(false, Ordering::Relaxed);
                         break;
                     },
@@ -248,7 +255,8 @@ impl RaknetSocket {
 
                 match sender.send(buf[..size].to_vec()).await{
                     Ok(_) => {},
-                    Err(_) => {
+                    Err(e) => {
+                        raknet_log!("channel send error : {}" , e);
                         connected_s.store(false, Ordering::Relaxed);
                         break;
                     },
@@ -288,6 +296,9 @@ impl RaknetSocket {
         let recvq = self.recvq.clone();
         let last_heartbeat_time = self.last_heartbeat_time.clone();
         let incomming_notify = self.incomming_notify.clone();
+        let s = self.s.clone();
+        let enable_loss = self.enable_loss.clone();
+        let loss_rate = self.loss_rate.clone();
         tokio::spawn(async move {
             loop{
                 if !connected.load(Ordering::Relaxed){
@@ -297,6 +308,7 @@ impl RaknetSocket {
                 let buf = match receiver.recv().await{
                     Some(buf) => buf,
                     None => {
+                        raknet_log!("channel receiver finished");
                         connected.store(false, Ordering::Relaxed);
                         break;
                     },
@@ -315,10 +327,10 @@ impl RaknetSocket {
                     let ack = read_packet_ack(&buf).await.unwrap();
                     for i in 0..ack.record_count{
                         if ack.sequences[i as usize].0 == ack.sequences[i as usize].1{
-                            sendq.ack(ack.sequences[i as usize].0);
+                            sendq.ack(ack.sequences[i as usize].0 , cur_timestamp_millis());
                         } else{
                             for i in ack.sequences[i as usize].0..ack.sequences[i as usize].1+1{
-                                sendq.ack(i);
+                                sendq.ack(i , cur_timestamp_millis());
                             }
                             
                         }
@@ -357,6 +369,7 @@ impl RaknetSocket {
 
                         for f in recvq.flush(&peer_addr){
                             if !RaknetSocket::handle(&f , &peer_addr ,&local_addr, &sendq, &user_data_sender , &incomming_notify).await.unwrap(){
+                                raknet_log!("handle error");
                                 connected.store(false, Ordering::Relaxed);
                                 is_break = true;
                                 break;
@@ -368,6 +381,19 @@ impl RaknetSocket {
                         }
                     }
 
+                    //flush ack
+                    let acks = recvq.get_ack();
+
+                    if acks.len() != 0 {
+    
+                        let packet = ACK{
+                            record_count: acks.len() as u16,
+                            sequences: acks,
+                        };
+    
+                        let buf = write_packet_ack(&packet).await.unwrap();
+                        RaknetSocket::sendto(&s , &buf, &peer_addr , &enable_loss  , &loss_rate).await.unwrap();
+                    }
                 } else {
                     raknet_log!("unknown packetid : {}", buf[0]);
                 }
@@ -389,24 +415,10 @@ impl RaknetSocket {
         let last_heartbeat_time = self.last_heartbeat_time.clone();
         tokio::spawn(async move {
             loop{
-                sleep(std::time::Duration::from_millis(50)).await;
-
-                // flush ack
-                let mut recvq = recvq.lock().await;
-                let acks = recvq.get_ack();
-
-                if acks.len() != 0 {
-
-                    let packet = ACK{
-                        record_count: acks.len() as u16,
-                        sequences: acks,
-                    };
-
-                    let buf = write_packet_ack(&packet).await.unwrap();
-                    RaknetSocket::sendto(&s , &buf, &peer_addr , &enable_loss  , &loss_rate).await.unwrap();
-                }
+                sleep(std::time::Duration::from_millis(SendQ::DEFAULT_TIMEOUT_MILLS as u64)).await;
 
                 // flush nack
+                let mut recvq = recvq.lock().await;
                 let nacks = recvq.get_nack();
                 if nacks.len() != 0{
                     let nack = NACK{
@@ -432,10 +444,11 @@ impl RaknetSocket {
 
                 //monitor log
                 if cur_timestamp_millis() - last_monitor_tick > 10000{
-                    raknet_log!("peer addr : {} , sendq size : {} , sentq size : {} , recvq size : {} ,  recvq fragment size : {} , ordered queue size : {} - {:?}" , 
+                    raknet_log!("peer addr : {} , sendq size : {} , sentq size : {} , rto : {} , recvq size : {} ,  recvq fragment size : {} , ordered queue size : {} - {:?}" , 
                         peer_addr,
                         sendq.get_reliable_queue_size(),
                         sendq.get_sent_queue_size(),
+                        sendq.get_rto(),
                         recvq.get_size(),
                         recvq.get_fragment_queue_size(),
                         recvq.get_ordered_packet(),
@@ -446,6 +459,7 @@ impl RaknetSocket {
 
                 // if exceed 60s not received any packet will close connection.
                 if cur_timestamp_millis() - last_heartbeat_time.load(Ordering::Relaxed) > RECEIVE_TIMEOUT{
+                    raknet_log!("recv timeout");
                     connected.store(false, Ordering::Relaxed);
                     break;
                 }
@@ -539,10 +553,9 @@ impl RaknetSocket {
             return Err(RaknetError::ConnectionClosed);
         }
 
-        self.sendq.lock().await.insert(r , buf)?;
-
         //flush sendq
         let mut sendq = self.sendq.lock().await;
+        sendq.insert(r , buf)?;
         for f in sendq.flush(cur_timestamp_millis(), &self.peer_addr){
             let data = f.serialize().await.unwrap();
             RaknetSocket::sendto(&self.s , &data, &self.peer_addr , &self.enable_loss , &self.loss_rate).await.unwrap();
@@ -602,7 +615,7 @@ impl RaknetSocket {
 
     /// Set the packet loss rate and use it for testing
     /// 
-    /// The `stage` parameter ranges from 1 to 10, indicating a packet loss rate of 90% to 0%.
+    /// The `stage` parameter ranges from 0 to 10, indicating a packet loss rate of 0% to 100%.
     /// # Example
     /// ```ignore
     /// let mut socket = RaknetSocket::connect("127.0.0.1:19132".parse().unwrap()).await.unwrap();
